package cn.ibizlab.assetmanagement.core.asset.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.asset.domain.EAMDocument;
import cn.ibizlab.assetmanagement.core.asset.filter.EAMDocumentSearchContext;
import cn.ibizlab.assetmanagement.core.asset.service.IEAMDocumentService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.asset.mapper.EAMDocumentMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[文档] 服务对象接口实现
 */
@Slf4j
@Service("EAMDocumentServiceImpl")
public class EAMDocumentServiceImpl extends ServiceImpl<EAMDocumentMapper, EAMDocument> implements IEAMDocumentService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetTypeService eamassettypeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMDocumentTypeService eamdocumenttypeService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMDocument et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamdocumentid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMDocument> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMDocument et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamdocumentid",et.getEamdocumentid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamdocumentid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMDocument> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMDocument get(String key) {
        EAMDocument et = getById(key);
        if(et==null){
            et=new EAMDocument();
            et.setEamdocumentid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMDocument getDraft(EAMDocument et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EAMDocument et) {
        return (!ObjectUtils.isEmpty(et.getEamdocumentid()))&&(!Objects.isNull(this.getById(et.getEamdocumentid())));
    }
    @Override
    @Transactional
    public boolean save(EAMDocument et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMDocument et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMDocument> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMDocument> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMDocument> selectByEamassettypeid(String eamassettypeid) {
        return baseMapper.selectByEamassettypeid(eamassettypeid);
    }

    @Override
    public void removeByEamassettypeid(String eamassettypeid) {
        this.remove(new QueryWrapper<EAMDocument>().eq("eamassettypeid",eamassettypeid));
    }

	@Override
    public List<EAMDocument> selectByEamdocumenttypeid(String eamdocumenttypeid) {
        return baseMapper.selectByEamdocumenttypeid(eamdocumenttypeid);
    }

    @Override
    public void removeByEamdocumenttypeid(String eamdocumenttypeid) {
        this.remove(new QueryWrapper<EAMDocument>().eq("eamdocumenttypeid",eamdocumenttypeid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMDocument> searchDefault(EAMDocumentSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMDocument> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMDocument>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EAMDocument et){
        //实体关系[DER1N_EAMDOCUMENT_EAMASSETTYPE_EAMASSETTYPEID]
        if(!ObjectUtils.isEmpty(et.getEamassettypeid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetType eamassettype=et.getEamassettype();
            if(ObjectUtils.isEmpty(eamassettype)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetType majorEntity=eamassettypeService.get(et.getEamassettypeid());
                et.setEamassettype(majorEntity);
                eamassettype=majorEntity;
            }
            et.setEamassettypename(eamassettype.getEamassettypename());
        }
        //实体关系[DER1N_EAMDOCUMENT_EAMDOCUMENTTYPE_EAMDOCUMENTTYPEID]
        if(!ObjectUtils.isEmpty(et.getEamdocumenttypeid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMDocumentType eamdocumenttype=et.getEamdocumenttype();
            if(ObjectUtils.isEmpty(eamdocumenttype)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMDocumentType majorEntity=eamdocumenttypeService.get(et.getEamdocumenttypeid());
                et.setEamdocumenttype(majorEntity);
                eamdocumenttype=majorEntity;
            }
            et.setEamdocumenttypename(eamdocumenttype.getEamdocumenttypename());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMDocument> getEamdocumentByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMDocument> getEamdocumentByEntities(List<EAMDocument> entities) {
        List ids =new ArrayList();
        for(EAMDocument entity : entities){
            Serializable id=entity.getEamdocumentid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



