package cn.ibizlab.assetmanagement.core.asset.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.asset.domain.EAMDocumentType;
import cn.ibizlab.assetmanagement.core.asset.filter.EAMDocumentTypeSearchContext;
import cn.ibizlab.assetmanagement.core.asset.service.IEAMDocumentTypeService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.asset.mapper.EAMDocumentTypeMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[文档类型] 服务对象接口实现
 */
@Slf4j
@Service("EAMDocumentTypeServiceImpl")
public class EAMDocumentTypeServiceImpl extends ServiceImpl<EAMDocumentTypeMapper, EAMDocumentType> implements IEAMDocumentTypeService {


    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMDocumentTypeService eamdocumenttypeService = this;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMDocumentService eamdocumentService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMDocumentType et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamdocumenttypeid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMDocumentType> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMDocumentType et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamdocumenttypeid",et.getEamdocumenttypeid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamdocumenttypeid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMDocumentType> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMDocumentType get(String key) {
        EAMDocumentType et = getById(key);
        if(et==null){
            et=new EAMDocumentType();
            et.setEamdocumenttypeid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMDocumentType getDraft(EAMDocumentType et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EAMDocumentType et) {
        return (!ObjectUtils.isEmpty(et.getEamdocumenttypeid()))&&(!Objects.isNull(this.getById(et.getEamdocumenttypeid())));
    }
    @Override
    @Transactional
    public boolean save(EAMDocumentType et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMDocumentType et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMDocumentType> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMDocumentType> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMDocumentType> selectByPeamdocumenttypeid(String eamdocumenttypeid) {
        return baseMapper.selectByPeamdocumenttypeid(eamdocumenttypeid);
    }

    @Override
    public void removeByPeamdocumenttypeid(String eamdocumenttypeid) {
        this.remove(new QueryWrapper<EAMDocumentType>().eq("peamdocumenttypeid",eamdocumenttypeid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMDocumentType> searchDefault(EAMDocumentTypeSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMDocumentType> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMDocumentType>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 顶级文档类型
     */
    @Override
    public Page<EAMDocumentType> searchTop(EAMDocumentTypeSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMDocumentType> pages=baseMapper.searchTop(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMDocumentType>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EAMDocumentType et){
        //实体关系[DER1N_EAMDOCUMENTTYPE_EAMDOCUMENTTYPE_PEAMDOCUMENTTYPEID]
        if(!ObjectUtils.isEmpty(et.getPeamdocumenttypeid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMDocumentType peamdocumenttype=et.getPeamdocumenttype();
            if(ObjectUtils.isEmpty(peamdocumenttype)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMDocumentType majorEntity=eamdocumenttypeService.get(et.getPeamdocumenttypeid());
                et.setPeamdocumenttype(majorEntity);
                peamdocumenttype=majorEntity;
            }
            et.setPeamdocumenttypename(peamdocumenttype.getEamdocumenttypename());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMDocumentType> getEamdocumenttypeByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMDocumentType> getEamdocumenttypeByEntities(List<EAMDocumentType> entities) {
        List ids =new ArrayList();
        for(EAMDocumentType entity : entities){
            Serializable id=entity.getEamdocumenttypeid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



